@@ -29,6 +29,29 @@ pub enum DependencyKind {
     Require,
 }
 
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub enum DynamicImportAssertion {
+    /// The value of this assertion could not be statically analyzed.
+    Unknown,
+    /// The value of this assertion is a statically analyzed string.
+    Known(String),
+}
+
+#[derive(Clone, Debug, Eq, PartialEq)]
+pub enum DynamicImportAssertions {
+    /// The set of assertion keys could not be statically analyzed.
+    Unknown,
+    /// The set of assertion keys is statically analyzed, though each respective
+    /// value may or may not not be.
+    Known(HashMap<String, DynamicImportAssertion>),
+}
+
+impl Default for DynamicImportAssertions {
+    fn default() -> Self {
+        DynamicImportAssertions::Known(HashMap::new())
+    }
+}
+
 #[derive(Clone, Debug, Eq, PartialEq)]
 pub struct DependencyDescriptor {
     pub kind: DependencyKind,
@@ -46,6 +69,9 @@ pub struct DependencyDescriptor {
     /// Import assertions for this dependency.
     /// NOTE: it's filled only for static imports and exports.
     pub import_assertions: HashMap<String, String>,
+    /// Import assertions for dynamic imports. Present if `is_dynamic` is `true`
+    /// and `kind` is not `DependencyKind::Require`.
+    pub dynamic_import_assertions: Option<DynamicImportAssertions>,
 }
 
 struct DependencyCollector<'a> {
@@ -80,6 +106,7 @@ impl<'a> Visit for DependencyCollector<'a> {
             specifier,
             specifier_span: node.src.span,
             import_assertions,
+            dynamic_import_assertions: None,
         });
     }
 
@@ -101,6 +128,7 @@ impl<'a> Visit for DependencyCollector<'a> {
                 specifier,
                 specifier_span: src.span,
                 import_assertions,
+                dynamic_import_assertions: None,
             });
         }
     }
@@ -117,6 +145,7 @@ impl<'a> Visit for DependencyCollector<'a> {
             specifier,
             specifier_span: node.src.span,
             import_assertions,
+            dynamic_import_assertions: None,
         });
     }
 
@@ -132,6 +161,7 @@ impl<'a> Visit for DependencyCollector<'a> {
             specifier,
             specifier_span: node.arg.span,
             import_assertions: HashMap::default(),
+            dynamic_import_assertions: None,
         });
     }
 
@@ -165,44 +195,17 @@ impl<'a> Visit for DependencyCollector<'a> {
 
         // import() are always dynamic, even if at top level
         let is_dynamic = !self.is_top_level || kind == DependencyKind::Import;
+        let dynamic_import_assertions = if kind == DependencyKind::Import {
+            Some(parse_dynamic_import_assertions(node.args.get(1)))
+        } else {
+            None
+        };
 
         if let Some(arg) = node.args.get(0) {
             if let Lit(lit) = &*arg.expr {
                 if let ast::Lit::Str(str_) = lit {
                     let specifier = str_.value.clone();
                     let leading_comments = self.get_leading_comments(node.span);
-                    let mut import_assertions = HashMap::default();
-
-                    if let Some(arg) = node.args.get(1) {
-                        if let Object(object_lit) = &*arg.expr {
-                            for prop in object_lit.props.iter() {
-                                if let ast::PropOrSpread::Prop(prop) = prop {
-                                    if let ast::Prop::KeyValue(key_value) = &**prop {
-                                        let maybe_key = match &key_value.key {
-                                            ast::PropName::Str(key) => Some(key.value.to_string()),
-                                            ast::PropName::Ident(ident) => {
-                                                Some(ident.sym.to_string())
-                                            }
-                                            _ => None,
-                                        };
-
-                                        if let Some(key) = maybe_key {
-                                            if key == "assert" {
-                                                import_assertions = if let Object(assertions_lit) =
-                                                    &*key_value.value
-                                                {
-                                                    parse_import_assertions(Some(&assertions_lit))
-                                                } else {
-                                                    HashMap::new()
-                                                };
-                                            }
-                                        }
-                                    }
-                                }
-                            }
-                        }
-                    }
-
                     self.items.push(DependencyDescriptor {
                         kind,
                         is_dynamic,
@@ -210,7 +213,8 @@ impl<'a> Visit for DependencyCollector<'a> {
                         span: node.span,
                         specifier,
                         specifier_span: str_.span,
-                        import_assertions,
+                        import_assertions: HashMap::new(),
+                        dynamic_import_assertions,
                     });
                 }
             }
@@ -247,6 +251,77 @@ fn parse_import_assertions(asserts: Option<&ast::ObjectLit>) -> HashMap<String,
     import_assertions
 }
 
+/// Parses import assertions from the second arg of a dynamic import.
+fn parse_dynamic_import_assertions(arg: Option<&ast::ExprOrSpread>) -> DynamicImportAssertions {
+    let mut assertions_map = HashMap::new();
+    if let Some(arg) = arg {
+        if arg.spread.is_some() {
+            return DynamicImportAssertions::Unknown;
+        }
+        if let ast::Expr::Object(object_lit) = &*arg.expr {
+            for prop in object_lit.props.iter() {
+                if let ast::PropOrSpread::Prop(prop) = prop {
+                    if let ast::Prop::KeyValue(key_value) = &**prop {
+                        let key = match &key_value.key {
+                            ast::PropName::Str(key) => key.value.to_string(),
+                            ast::PropName::Ident(ident) => ident.sym.to_string(),
+                            _ => return DynamicImportAssertions::Unknown,
+                        };
+                        if key == "assert" {
+                            if let ast::Expr::Object(assertions_lit) = &*key_value.value {
+                                for prop in assertions_lit.props.iter() {
+                                    if let ast::PropOrSpread::Prop(prop) = prop {
+                                        if let ast::Prop::KeyValue(key_value) = &**prop {
+                                            let key = match &key_value.key {
+                                                ast::PropName::Str(key) => key.value.to_string(),
+                                                ast::PropName::Ident(ident) => {
+                                                    ident.sym.to_string()
+                                                }
+                                                _ => return DynamicImportAssertions::Unknown,
+                                            };
+                                            if let ast::Expr::Lit(value_lit) = &*key_value.value {
+                                                if let ast::Lit::Str(str_) = value_lit {
+                                                    assertions_map.insert(
+                                                        key,
+                                                        DynamicImportAssertion::Known(
+                                                            str_.value.to_string(),
+                                                        ),
+                                                    );
+                                                } else {
+                                                    assertions_map.insert(
+                                                        key,
+                                                        DynamicImportAssertion::Unknown,
+                                                    );
+                                                }
+                                            } else {
+                                                assertions_map
+                                                    .insert(key, DynamicImportAssertion::Unknown);
+                                            }
+                                        } else {
+                                            return DynamicImportAssertions::Unknown;
+                                        }
+                                    } else {
+                                        return DynamicImportAssertions::Unknown;
+                                    }
+                                }
+                            } else {
+                                return DynamicImportAssertions::Unknown;
+                            }
+                        }
+                    } else {
+                        return DynamicImportAssertions::Unknown;
+                    }
+                } else {
+                    return DynamicImportAssertions::Unknown;
+                }
+            }
+        } else {
+            return DynamicImportAssertions::Unknown;
+        }
+    }
+    DynamicImportAssertions::Known(assertions_map)
+}
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -342,6 +417,7 @@ try {
                     specifier: JsWord::from("./test.ts"),
                     specifier_span: Span::new(BytePos(21), BytePos(32), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::ImportType,
@@ -355,6 +431,7 @@ try {
                     specifier: JsWord::from("./foo.d.ts"),
                     specifier_span: Span::new(BytePos(72), BytePos(84), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Export,
@@ -368,6 +445,7 @@ try {
                     specifier: JsWord::from("./buzz.ts"),
                     specifier_span: Span::new(BytePos(136), BytePos(147), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::ExportType,
@@ -388,6 +466,7 @@ try {
                     specifier: JsWord::from("./fizz.d.ts"),
                     specifier_span: Span::new(BytePos(206), BytePos(219), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Require,
@@ -397,6 +476,7 @@ try {
                     specifier: JsWord::from("path"),
                     specifier_span: Span::new(BytePos(246), BytePos(252), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Import,
@@ -406,6 +486,7 @@ try {
                     specifier: JsWord::from("./foo1.ts"),
                     specifier_span: Span::new(BytePos(280), BytePos(291), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: Some(Default::default()),
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Import,
@@ -415,6 +496,7 @@ try {
                     specifier: JsWord::from("./foo.ts"),
                     specifier_span: Span::new(BytePos(330), BytePos(340), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: Some(Default::default()),
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Require,
@@ -424,6 +506,7 @@ try {
                     specifier: JsWord::from("some_package"),
                     specifier_span: Span::new(BytePos(402), BytePos(416), Default::default()),
                     import_assertions: HashMap::default(),
+                    dynamic_import_assertions: None,
                 }
             ]
         );
@@ -437,14 +520,32 @@ export { bar } from "./test.json" assert { "type": "json" };
 import foo from "./foo.json" assert { type: "json" };
 const fizz = await import("./fizz.json", { "assert": { type: "json" } });
 const buzz = await import("./buzz.json", { assert: { "type": "json" } });
+const d1 = await import("./d1.json");
+const d2 = await import("./d2.json", {});
+const d3 = await import("./d3.json", bar);
+const d4 = await import("./d4.json", { assert: {} });
+const d5 = await import("./d5.json", { assert: bar });
+const d6 = await import("./d6.json", { assert: {}, ...bar });
+const d7 = await import("./d7.json", { assert: {}, ["assert"]: "bad" });
+const d8 = await import("./d8.json", { assert: { type: bar } });
+const d9 = await import("./d9.json", { assert: { type: "json", ...bar } });
+const d10 = await import("./d10.json", { assert: { type: "json", ["type"]: "bad" } });
       "#;
         let (module, comments) = helper("test.ts", &source).unwrap();
         let mut expected_assertions1 = HashMap::new();
         expected_assertions1.insert("type".to_string(), "typescript".to_string());
         let mut expected_assertions2 = HashMap::new();
         expected_assertions2.insert("type".to_string(), "json".to_string());
+        let dynamic_expected_assertions2 = DynamicImportAssertions::Known({
+            let mut map = HashMap::new();
+            map.insert(
+                "type".to_string(),
+                DynamicImportAssertion::Known("json".to_string()),
+            );
+            map
+        });
         let dependencies = analyze_dependencies(&module, &comments);
-        assert_eq!(dependencies.len(), 6);
+        assert_eq!(dependencies.len(), 16);
         assert_eq!(
             dependencies,
             vec![
@@ -456,6 +557,7 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     specifier: JsWord::from("./test.ts"),
                     specifier_span: Span::new(BytePos(21), BytePos(32), Default::default()),
                     import_assertions: expected_assertions1.clone(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Export,
@@ -465,6 +567,7 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     specifier: JsWord::from("./test.ts"),
                     specifier_span: Span::new(BytePos(80), BytePos(91), Default::default()),
                     import_assertions: expected_assertions1,
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Export,
@@ -474,6 +577,7 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     specifier: JsWord::from("./test.json"),
                     specifier_span: Span::new(BytePos(145), BytePos(158), Default::default()),
                     import_assertions: expected_assertions2.clone(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Import,
@@ -483,6 +587,7 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     specifier: JsWord::from("./foo.json"),
                     specifier_span: Span::new(BytePos(202), BytePos(214), Default::default()),
                     import_assertions: expected_assertions2.clone(),
+                    dynamic_import_assertions: None,
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Import,
@@ -491,7 +596,8 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     span: Span::new(BytePos(259), BytePos(312), Default::default()),
                     specifier: JsWord::from("./fizz.json"),
                     specifier_span: Span::new(BytePos(266), BytePos(279), Default::default()),
-                    import_assertions: expected_assertions2.clone(),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(dynamic_expected_assertions2.clone()),
                 },
                 DependencyDescriptor {
                     kind: DependencyKind::Import,
@@ -500,7 +606,112 @@ const buzz = await import("./buzz.json", { assert: { "type": "json" } });
                     span: Span::new(BytePos(333), BytePos(386), Default::default()),
                     specifier: JsWord::from("./buzz.json"),
                     specifier_span: Span::new(BytePos(340), BytePos(353), Default::default()),
-                    import_assertions: expected_assertions2,
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(dynamic_expected_assertions2.clone()),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(405), BytePos(424), Default::default()),
+                    specifier: JsWord::from("./d1.json"),
+                    specifier_span: Span::new(BytePos(412), BytePos(423), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Known(HashMap::new())),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(443), BytePos(466), Default::default()),
+                    specifier: JsWord::from("./d2.json"),
+                    specifier_span: Span::new(BytePos(450), BytePos(461), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Known(HashMap::new())),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(485), BytePos(509), Default::default()),
+                    specifier: JsWord::from("./d3.json"),
+                    specifier_span: Span::new(BytePos(492), BytePos(503), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(528), BytePos(563), Default::default()),
+                    specifier: JsWord::from("./d4.json"),
+                    specifier_span: Span::new(BytePos(535), BytePos(546), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Known(HashMap::new())),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(582), BytePos(618), Default::default()),
+                    specifier: JsWord::from("./d5.json"),
+                    specifier_span: Span::new(BytePos(589), BytePos(600), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(637), BytePos(680), Default::default()),
+                    specifier: JsWord::from("./d6.json"),
+                    specifier_span: Span::new(BytePos(644), BytePos(655), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(699), BytePos(753), Default::default()),
+                    specifier: JsWord::from("./d7.json"),
+                    specifier_span: Span::new(BytePos(706), BytePos(717), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(772), BytePos(818), Default::default()),
+                    specifier: JsWord::from("./d8.json"),
+                    specifier_span: Span::new(BytePos(779), BytePos(790), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Known({
+                        let mut map = HashMap::new();
+                        map.insert("type".to_string(), DynamicImportAssertion::Unknown);
+                        map
+                    })),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(837), BytePos(894), Default::default()),
+                    specifier: JsWord::from("./d9.json"),
+                    specifier_span: Span::new(BytePos(844), BytePos(855), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
+                },
+                DependencyDescriptor {
+                    kind: DependencyKind::Import,
+                    is_dynamic: true,
+                    leading_comments: Vec::new(),
+                    span: Span::new(BytePos(914), BytePos(981), Default::default()),
+                    specifier: JsWord::from("./d10.json"),
+                    specifier_span: Span::new(BytePos(921), BytePos(933), Default::default()),
+                    import_assertions: HashMap::new(),
+                    dynamic_import_assertions: Some(DynamicImportAssertions::Unknown),
                 },
             ]
         );
